/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

// Tableau redimensionnable (contenant des entiers)

#include "prelude.h"

#include "vector.h"

struct Vector {
  int capacity;
  int *data;    // tableau de taille capacity
  int size;     // invariant 0 <= size <= capacity
};

vector *vector_create(int capacity) {
  vector *v = malloc(sizeof(struct Vector));
  v->capacity = capacity;
  v->data = calloc(capacity, sizeof(int));
  v->size = 0;
  return v;
}

int vector_size(vector *v) {
  return v->size;
}

void vector_resize(vector *v, int s) {
  assert(0 <= s);
  if (s > v->capacity) {
    // il faut agrandir => on double la capacité
    v->capacity = 2 * v->capacity;
    if (v->capacity < s) { v->capacity = s; }
    int *old = v->data;
    v->data = calloc(v->capacity, sizeof(int));
    for (int i = 0; i < v->size; i++) {
      v->data[i] = old[i];
    }
    free(old);
  } else if (s < v->capacity / 4) {
    // on peut rétrécir => on divise par deux la capacité
    v->capacity = v->capacity / 2;
    int *old = v->data;
    v->data = calloc(v->capacity, sizeof(int));
    for (int i = 0; i < s; i++) {
      v->data[i] = old[i];
    }
    free(old);
  }
  v->size = s;
}

int vector_get(vector *v, int i) {
  assert(0 <= i && i < v->size);
  return v->data[i];
}

void vector_set(vector *v, int i, int x) {
  assert(0 <= i && i < v->size);
  v->data[i] = x;
}

void vector_push(vector *v, int x) {
  int n = vector_size(v);
  vector_resize(v, n+1);
  vector_set(v, n, x);
}
int vector_top(vector *v) {
  int n = vector_size(v) - 1;
  assert(0 <= n);
  return vector_get(v, n);
}
int vector_pop(vector *v) {
  int n = vector_size(v) - 1;
  assert(0 <= n);
  int r = vector_get(v, n);
  vector_resize(v, n);
  return r;
}

void vector_delete(vector *v) {
  free(v->data);
  free(v);
}
